@@ -178,7 +178,7 @@ def prepDfTas(dfTa, dfKS, se_tissue_annotation, tissue=None):
 
     return dfTas[['SYNERGY_SCORE', 'Tijk']]
 
-def split_train_test_validate_predict(df, factor=1/2, random_state=None):
+def split_train_test_validate_predict(df, factor=1/2, random_state=None, stratified=False):
     
     print()
     print('All pairs:\t\t', df.shape[0])
@@ -187,9 +187,15 @@ def split_train_test_validate_predict(df, factor=1/2, random_state=None):
     df_temp = df.loc[(~df['Cij'].isna()) & (~df['Sik'].isna()) & (~df['Sjk'].isna()) & (~df['SYNERGY_SCORE'].isna())]
     print('All with known pairs:\t', df_temp.shape[0])
 
-    print('Sampling training pairs')
-    se_temp = pd.Series(df_temp.reset_index()[['MODEL', 'DRUG1', 'DRUG2']].apply(lambda s: s[0] + '_' + '_'.join(np.sort([s[1], s[2]])), axis=1))
-    ind_temp = pd.Index(se_temp).isin(pd.Series(se_temp.unique()).sample(n=int(pd.Series(se_temp.unique()).shape[0] * factor), random_state=random_state).values)
+    if not stratified:
+        print('Sampling training pairs. Random')
+        se_temp = pd.Series(df_temp.reset_index()[['MODEL', 'DRUG1', 'DRUG2']].apply(lambda s: s[0] + '_' + '_'.join(np.sort([s[1], s[2]])), axis=1))
+        ind_temp = pd.Index(se_temp).isin(pd.Series(se_temp.unique()).sample(n=int(pd.Series(se_temp.unique()).shape[0] * factor), random_state=random_state).values)
+    else:
+        print('Sampling training pairs. Stratified by drug pairs')
+        se_temp = pd.Series(df_temp.reset_index()[['DRUG1', 'DRUG2']].apply(lambda s: '_'.join(np.sort([s[0], s[1]])), axis=1))
+        ind_temp = pd.Index(se_temp).isin(pd.Series(se_temp.unique()).sample(n=int(pd.Series(se_temp.unique()).shape[0] * factor), random_state=random_state).values)
+
     df_train_test = df_temp.loc[ind_temp]
     #df_train_test = df_temp.sample(n=int(df_temp.shape[0] * factor), random_state=random_state)
     print('Training-testing pairs:\t', df_train_test.shape[0])
@@ -383,13 +389,19 @@ def trainOneTestAnother(df_one, df_another, n=10, n_sample=0.5):
         print(dfTas2.shape)
 
         print('\n', 'M:')
-        res_temp[('EN', i)] = testCase(dfTas1, dfTas2, encode=True, useAllFeatures=False, cv=None)
+        res_temp[('EN', i)] = testCase(dfTas1, dfTas2, encode=True, useAllFeatures=False, cv=None)[0]
         print('\n', 'ACDA:')
-        res_temp[('ACDA', i)] = testCase(dfTas1, dfTas2, encode=False, useAllFeatures=False, cv=None)
+        res_temp[('ACDA', i)], df_A = testCase(dfTas1, dfTas2, encode=False, useAllFeatures=False, cv=None)
         print('\n', 'CDA:')
-        res_temp[('CDA', i)] = testCase(dfTas1, dfTas2, encode=False, useAllFeatures=False, cv=None, clf=LinearRegression())
+        res_temp[('CDA', i)] = testCase(dfTas1, dfTas2, encode=False, useAllFeatures=False, cv=None, clf=LinearRegression())[0]
         print('\n', 'EN-ACDA:')
-        res_temp[('EN-ACDA', i)] = testCase(dfTas1, dfTas2, encode=True, useAllFeatures=True, cv=None)
+        res_temp[('EN-ACDA', i)], df_B = testCase(dfTas1, dfTas2, encode=True, useAllFeatures=True, cv=None)
+
+        print('\n', 'ACDA and EN-ACDA averaged:')
+        df_AB = (df_A + df_B)/2.
+        av_score = np.round(pearsonr(df_AB['SYNERGY_SCORE'].astype(float), df_AB['Predicted'].astype(float))[0], 3)
+        res_temp[('ACDA-EN-ACDA', i)] = {'self': np.nan, 'cv_mean': np.nan, 'cv_std': np.nan, 'val': av_score, 'ro':  np.nan}
+        print(av_score)
 
     df_temp = pd.DataFrame(res_temp).T['val']
     df_temp2 = pd.concat([df_temp.groupby(level=0).mean(), df_temp.groupby(level=0).sem()], axis=1)
@@ -487,7 +499,7 @@ def testCase(df_train_test, df_validate, CDA_features=['Tijk', 'Cij', 'Sik', 'Sj
             features = df_train_test.columns[~df_train_test.columns.isin(['SYNERGY_SCORE'] + CDA_features)]
     else:
         features = CDA_features
-        
+
     r, res = fit_validate_predict(df_train_test[features].values, df_train_test['SYNERGY_SCORE'].values,
                          extData=df_validate[features].values, extSynergy=df_validate['SYNERGY_SCORE'].values,
                          cv=cv, clf=clf)
@@ -501,9 +513,9 @@ def testCase(df_train_test, df_validate, CDA_features=['Tijk', 'Cij', 'Sik', 'Sj
     except:
         res.update({'ro': np.nan})
 
-    return res
+    return res, df_ro
 
-def MonteCarloCrossValidation(dfTas, n=10, sample_non_synergy=False, sample_non_synergy_size=100, clf_for_CDA=LogisticRegression(), deidentify=False, factor=2/3):
+def MonteCarloCrossValidation(dfTas, n=10, sample_non_synergy=False, sample_non_synergy_size=100, clf_for_CDA=LogisticRegression(), deidentify=False, factor=2/3, stratified=False):
     
     '''For continuous predictee: clf_for_CDA=LinearRegression()
     '''
@@ -517,7 +529,7 @@ def MonteCarloCrossValidation(dfTas, n=10, sample_non_synergy=False, sample_non_
         if sample_non_synergy:
             dfTas_copy.loc[dfTas_copy[dfTas_copy['SYNERGY_SCORE'].isna()].sample(sample_non_synergy_size, random_state=i).index, 'SYNERGY_SCORE'] = 0.
             
-        df_train_test, df_validate, df_predict = split_train_test_validate_predict(dfTas_copy, factor=factor, random_state=i)
+        df_train_test, df_validate, df_predict = split_train_test_validate_predict(dfTas_copy, factor=factor, random_state=i, stratified=stratified)
         
         if deidentify:
             f = df_validate.index.to_frame()
@@ -526,16 +538,22 @@ def MonteCarloCrossValidation(dfTas, n=10, sample_non_synergy=False, sample_non_
             df_validate.index = pd.MultiIndex.from_frame(f)
 
         print('\n', 'ACDA:')
-        res_temp[('ACDA', i)] = testCase(df_train_test, df_validate, encode=False, useAllFeatures=False, cv=None)
+        res_temp[('ACDA', i)], df_A = testCase(df_train_test, df_validate, encode=False, useAllFeatures=False, cv=None)
         
         print('\n', 'CDA:')
-        res_temp[('CDA', i)] = testCase(df_train_test, df_validate, encode=False, useAllFeatures=False, cv=None, clf=LinearRegression())
+        res_temp[('CDA', i)] = testCase(df_train_test, df_validate, encode=False, useAllFeatures=False, cv=None, clf=LinearRegression())[0]
 
         print('\n', 'EN:')
-        res_temp[('EN', i)] = testCase(df_train_test, df_validate, encode=True, useAllFeatures=False, cv=None)
+        res_temp[('EN', i)] = testCase(df_train_test, df_validate, encode=True, useAllFeatures=False, cv=None)[0]
         
         print('\n', 'EN-ACDA:')
-        res_temp[('EN-ACDA', i)] = testCase(df_train_test, df_validate, encode=True, useAllFeatures=True, cv=None)
+        res_temp[('EN-ACDA', i)], df_B = testCase(df_train_test, df_validate, encode=True, useAllFeatures=True, cv=None)
+
+        print('\n', 'ACDA and EN-ACDA averaged:')
+        df_AB = (df_A + df_B)/2.
+        av_score = np.round(pearsonr(df_AB['SYNERGY_SCORE'].astype(float), df_AB['Predicted'].astype(float))[0], 3)
+        res_temp[('ACDA-EN-ACDA', i)] = {'self': np.nan, 'cv_mean': np.nan, 'cv_std': np.nan, 'val': av_score, 'ro':  np.nan}
+        print(av_score)
 
     df_temp1 = pd.DataFrame(res_temp).T.sort_index()
     df_temp2 = pd.concat([df_temp1.groupby(level=0).mean().round(3),
@@ -544,7 +562,7 @@ def MonteCarloCrossValidation(dfTas, n=10, sample_non_synergy=False, sample_non_
     
     return df_temp2, df_temp1
 
-def downsampleRun(ra, dfTas=None, pref='temp', mid='temp', rep=10, basedir='output/', cv=None):
+def downsampleRun(ra, dfTas=None, pref='temp', mid='temp', rep=10, basedir='output/', cv=None, stratified=False):
     
     '''df_sub10000 = downsampleRun(range(500, 10000, 500), dfTas=dfTas_AZ_breast, pref='', mid='AZ_breast')
     '''
@@ -559,21 +577,21 @@ def downsampleRun(ra, dfTas=None, pref='temp', mid='temp', rep=10, basedir='outp
             if not os.path.isfile(fname):
                 print(n, i)
                 res = dict()
-                df_train_test, df_validate, df_predict = split_train_test_validate_predict(dfTas, factor=2/3, random_state=i)
+                df_train_test, df_validate, df_predict = split_train_test_validate_predict(dfTas, factor=2/3, random_state=i, stratified=stratified)
 
                 df_train_test_sample = df_train_test.sample(n, random_state=i)
 
                 print('\n', 'EN:')
-                res.update({('EN', n, i): testCase(df_train_test_sample, df_validate, encode=True, useAllFeatures=False, cv=None)})
+                res.update({('EN', n, i): testCase(df_train_test_sample, df_validate, encode=True, useAllFeatures=False, cv=None)[0]})
 
                 print('\n', 'ACDA:')
-                res.update({('ACDA', n, i): testCase(df_train_test_sample, df_validate, encode=False, useAllFeatures=False, cv=None)})
+                res.update({('ACDA', n, i): testCase(df_train_test_sample, df_validate, encode=False, useAllFeatures=False, cv=None)[0]})
 
                 print('\n', 'CDA:')
-                res.update({('CDA', n, i): testCase(df_train_test_sample, df_validate, encode=False, useAllFeatures=False, cv=None, clf=LinearRegression())})
+                res.update({('CDA', n, i): testCase(df_train_test_sample, df_validate, encode=False, useAllFeatures=False, cv=None, clf=LinearRegression())[0]})
 
                 print('\n', 'EN-ACDA:')
-                res.update({('EN-ACDA', n, i): testCase(df_train_test_sample, df_validate, encode=True, useAllFeatures=True, cv=None)})
+                res.update({('EN-ACDA', n, i): testCase(df_train_test_sample, df_validate, encode=True, useAllFeatures=True, cv=None)[0]})
 
                 with open(fname, 'wb') as outfile:
                     pickle.dump(res, outfile)
